<?php

namespace Drupal\revision_delete_tools\Drush\Commands;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\TranslatableInterface;
use Drush\Commands\DrushCommands;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drush\Attributes as CLI;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Queue\QueueFactory;

/**
 * Command to queue entities for revision removal.
 */
final class RemoveRevisionsCommands extends DrushCommands {

  /**
   * The number of revisions to keep.
   */
  protected int $revisionsToKeep = 3;

  public function __construct(
    private readonly EntityTypeManagerInterface $entityTypeManager,
    private readonly QueueFactory $queueFactory,
  ) {
    parent::__construct();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('queue'),
    );
  }

  /**
   * Queues revisions for removal.
   */
  #[CLI\Command(name: 'rdt:remove-revisions')]
  #[CLI\Argument(name: 'entityType', description: 'The entity type (e.g. node, media).')]
  #[CLI\Argument(name: 'bundle', description: 'The bundle name (e.g. page for nodes, image for media).')]
  #[CLI\Argument(name: 'entityId', description: 'The specific entity ID (optional).')]
  #[CLI\Option(name: 'keep', description: 'The number of revisions to keep (optional, default is 3).')]
  #[CLI\Usage(name: 'drush rdt:remove-revisions', description: 'Queue revisions for all entities of the specified type and bundle.')]
  #[CLI\Usage(name: 'drush rdt:remove-revisions node page --keep=5', description: 'Queue revisions for all "page" nodes and keep 5 revisions.')]
  #[CLI\Usage(name: 'drush rdt:remove-revisions node page 123 --keep=3', description: 'Queue revisions for node with ID 123 under "page" and keep 3 revisions.')]
  public function queueRemoveRevisions(string $entityType, string $bundle, ?string $entityId = NULL, int $keep = 3): void {
    $this->revisionsToKeep = $keep;

    if ($entityId === NULL) {
      $this->queueRevisionsForEntitiesByBundle($entityType, $bundle);
    }
    else {
      $this->queueRevisionsForEntity($entityType, $bundle, $entityId);
    }

    $this->output()->writeln("Entities have been queued for revision removal.");
  }

  /**
   * Queues revisions for all entities of a specific type and bundle.
   */
  protected function queueRevisionsForEntitiesByBundle(string $entityType, string $bundle): void {
    $storage = $this->entityTypeManager->getStorage($entityType);
    $entityTypeDefinition = $storage->getEntityType();
    $bundleKey = $entityTypeDefinition->getKey('bundle') ?: 'type';

    $entityIds = $storage->getQuery()
      ->accessCheck(FALSE)
      ->condition($bundleKey, $bundle)
      ->execute();

    if ($entityIds === []) {
      $this->output()->writeln("<error>No entities found for type '$entityType' and bundle '$bundle'.</error>");
      return;
    }

    foreach ($entityIds as $entityId) {
      $this->queueRevisionsForEntity($entityType, $bundle, $entityId);
    }
  }

  /**
   * Queues a specific entity's revisions for removal.
   */
  protected function queueRevisionsForEntity(string $entityType, string $bundle, string $entityId): void {
    $vids = $this->getRevisionIds($entityType, $entityId);

    if ($vids === []) {
      return;
    }

    // For each revision, create a separate queue item.
    foreach ($vids as $vid) {
      $this->queueFactory->get('remove_revisions')->createItem([
        'bundle' => $bundle,
        'entityId' => $entityId,
        'entityType' => $entityType,
        'keep' => $this->revisionsToKeep,
        'vid' => $vid,
      ]);
    }
  }

  /**
   * Get the revisions to remove for a specific entity.
   */
  protected function getRevisionIds(string $entityType, string $entityId): array {
    $storage = $this->entityTypeManager->getStorage($entityType);
    $entity = $storage->load($entityId);
    if (!$entity) {
      $this->output()->writeln("<error>Entity not found: $entityId</error>");
      return [];
    }

    $entityTypeDefinition = $entity->getEntityType();
    if (!$entityTypeDefinition->isRevisionable()) {
      $this->output()->writeln("<error>Entity type is not revisionable: $entityType</error>");
      return [];
    }

    $vids = [];
    if ($entity instanceof TranslatableInterface) {
      foreach ($entity->getTranslationLanguages() as $language) {
        $vids = array_merge($vids, $this->getRevisions($entity, $language->getId()));
      }
    }
    else {
      $vids = $this->getRevisions($entity, NULL);
    }

    return $vids;
  }

  /**
   * Get the revision IDs for a specific entity translation.
   */
  protected function getRevisions(EntityInterface $entity, ?string $language): array {
    $storage = $this->entityTypeManager->getStorage($entity->getEntityTypeId());
    $idColumn = $entity->getEntityTypeId() === 'media' ? 'mid' : 'nid';

    // Query to fetch revisions beyond the number to keep.
    $query = $storage->getQuery()
      ->accessCheck(FALSE)
      ->allRevisions()
      ->condition($idColumn, $entity->id())
      ->range($this->revisionsToKeep, PHP_INT_MAX)
      ->sort('vid', 'DESC');

    // If a language filter is specified, add it.
    if ($language) {
      $query->condition('langcode', $language);
    }

    return array_keys($query->execute());
  }

}
