<?php

namespace Drupal\revision_delete_tools\Plugin\QueueWorker;

use Drupal\Core\Queue\QueueWorkerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Entity\RevisionableStorageInterface;

/**
 * A Queue Worker that removes revisions for a given entity.
 *
 * @QueueWorker(
 *   id = "remove_revisions",
 *   title = @Translation("Remove entity revisions"),
 *   cron = {"time" = 60}
 * )
 */
final class RemoveRevisions extends QueueWorkerBase implements ContainerFactoryPluginInterface {
  /**
   * The logger service.
   */
  protected LoggerChannelInterface $logger;

  /**
   * The default number of revisions to keep.
   */
  protected const DEFAULT_REVISIONS_TO_KEEP = 3;

  /**
   * Constructs a new RemoveRevisions instance.
   *
   * @param array<string, mixed> $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the queue worker.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger
   *   The logger factory service.
   */
  public function __construct(
    array $configuration,
    string $plugin_id,
    mixed $plugin_definition,
    private readonly EntityTypeManagerInterface $entityTypeManager,
    LoggerChannelFactoryInterface $logger,
  ) {
    $this->logger = $logger->get('njcourts_general');
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new self(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('logger.factory'),
    );
  }

  /**
   * Processes one queue item to remove a single revision from an entity.
   */
  public function processItem(mixed $data): void {
    $storage = $this->entityTypeManager->getStorage($data['entityType']);

    if (!($storage instanceof RevisionableStorageInterface)) {
      $this->logger->warning('Could not load storage for entity type @type', [
        '@type' => $data['entityType'],
      ]);
      return;
    }

    $storage->deleteRevision($data['vid']);
  }

}
